/*
    CoolSetup installation system
    Copyright (C) 2001-2002 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../shared/instclass.h"
#include "./instclass.h"
#include "../shared/datatypes.h"

int installfile::addfiletozip(zipFile zf, char *path, char *filenameinzip/*, HWND progress*/)
{
	FILE * fin;
	int size_read, err;
	zip_fileinfo zi;
	void *buf=malloc(8192);
	int opt_compress_level=compressionstrength;
	int size_buf=8192;
	
	zi.tmz_date.tm_sec = zi.tmz_date.tm_min = zi.tmz_date.tm_hour = zi.tmz_date.tm_mday = zi.tmz_date.tm_min = zi.tmz_date.tm_year = 0;
	zi.dosDate = 0;
	zi.internal_fa = 0;
	zi.external_fa = 0;
	filetime(path,&zi.tmz_date,&zi.dosDate);

	err = zipOpenNewFileInZip(zf,filenameinzip,&zi, NULL,0,NULL,0,NULL, (opt_compress_level != 0) ? Z_DEFLATED : 0, opt_compress_level);

	fin = fopen(path,"rb");
	if (fin==NULL) {
		err=ZIP_ERRNO;
		MsgBox(MB_OK|MB_ICONWARNING, "Error in opening '%s' for reading\n", path);
	}

	if (err == ZIP_OK) {
		/*fseek(fin, 0, SEEK_END);
		SendMessage(progress, PBM_SETRANGE, 0, (LPARAM)ftell(fin)/size_buf);
		SendMessage(progress, PBM_SETSTEP, (WPARAM)1, 0);
		SendMessage(progress, PBM_SETPOS, (WPARAM)0, 0);	
		fseek(fin, 0, SEEK_SET);*/

		do {
			err = ZIP_OK;
			size_read = fread(buf,1,size_buf,fin);
			//SendMessage(progress, PBM_STEPIT, 0, 0);
			if (size_read < size_buf)
				if (feof(fin)==0) {
					MsgBox(MB_OK|MB_ICONWARNING, "Error in reading '%s'\n",filenameinzip);
					err = ZIP_ERRNO;
				}
				
				if (size_read>0) {
					err = zipWriteInFileInZip (zf,buf,size_read);
					if (err<0) {
						MsgBox(MB_OK|MB_ICONWARNING, "Error in writing '%s' in the zipfile\n", filenameinzip);
					}
					
				}
		} while ((err == ZIP_OK) && (size_read>0));
	}	

	if (fin)
		fclose(fin);
	
	if (err<0)
		err=ZIP_ERRNO;
	else
	{                    
		err = zipCloseFileInZip(zf);
		if (err!=ZIP_OK)
			MsgBox(MB_OK|MB_ICONWARNING, "Error in closing '%s' in the zipfile\n", filenameinzip);
	}
	free(buf);
	return 0;
}

installfile::installfile()
{
}

installfile::~installfile()
{
}

int installfile::compile(HWND progress)
{
	long i, j, c=0;
	FILE *file, *file2;
	char buf[_MAX_PATH], tmppath[_MAX_PATH];
	char buf1[1024], buf2[1024];
	zipFile zf;
	//char inzippath[1024];
	fileconvnfo fci;

	for (i=0; i<fgno; i++)
		c+=fg[i]->fgino;
	c=c*2;
	c+=50;

	SendMessage(progress, PBM_SETRANGE, 0, MAKELPARAM(0, c));
	SendMessage(progress, PBM_SETSTEP, (WPARAM)1, 0);
	cs_get_progpath(buf);
	strcat(buf, tgtplatform);
	file=fopen(buf, "rb");
	if (file==NULL) {
		MsgBox(MB_OK, "Bad platform (installer) selected! Please reselect and then start compling again...");
		return 1;
	}
	fclose(file);
	if (!strcmpi(outputpath, "")) {
		MsgBox(MB_OK|MB_ICONSTOP, "You must specify a valid output path! (currently '%s')", outputpath);
		SendMessage(progress, PBM_SETPOS, (WPARAM)0, 0);
		return 0;
	}
	if (save(projpath)) {
		SendMessage(progress, PBM_SETPOS, (WPARAM)0, 0);
		MsgBox(MB_OK|MB_ICONSTOP, "There was an error during saving project file '%s'!", projpath);
		return 0;
	}
	strcpy(tmppath, outputpath);
	strcat(tmppath, ".zip");
	zf=zipOpen(tmppath, 0);

	for (i=0; i<fgno; i++)
		for (j=0; j<fg[i]->fgino; j++)
			fci.additem(fg[i]->fgi[j]->source);
	for (i=0; i<fci.count; i++) {
		if (!fci.getitem(i, buf1, buf2))
			addfiletozip(zf, buf1, buf2/*, progress2*/);
		SendMessage(progress, PBM_STEPIT, 0, 0);
	}
	GetTempPath(_MAX_PATH, tmppath);
	for (i=0; i<fgno; i++) {
		for (j=0; j<fg[i]->fgino; j++) {
			if (!fci.getitem(fci.getitempos(fg[i]->fgi[j]->source), buf1, buf2))
				strcpy(fg[i]->fgi[j]->source, buf2);
			SendMessage(progress, PBM_STEPIT, 0, 0);
		}
	}
	strcat(tmppath, "coolsetup.cfg");
	if (save(tmppath)) {
		load(projpath);
		MsgBox(MB_OK|MB_ICONSTOP, "There was an error during saving temporary file '%s'!", tmppath);
		SendMessage(progress, PBM_SETPOS, (WPARAM)0, 0);
		return 0;
	}
	addfiletozip(zf, tmppath, "coolsetup.cfg"/*, progress2*/);

	if (showreadme)
		addfiletozip(zf, readmefile, "readme.csf"/*, progress2*/);

	if (showlicense)
		addfiletozip(zf, licensefile, "license.csf"/*, progress2*/);

	SendMessage(progress, PBM_SETSTEP, (WPARAM)20, 0);
	SendMessage(progress, PBM_STEPIT, 0, 0);
	if (!DeleteFile(tmppath))
		MsgBox(MB_OK|MB_ICONWARNING, "Can't remove temporary file '%s'", tmppath);

	zipClose(zf, "Made by CoolSetup - http://coolsetup.sf.net/");

	file=fopen(buf, "rb");
	file2=fopen(outputpath, "wb");
	if (file==NULL) {
		if (file2!=NULL)
			fclose(file2);
		MsgBox(MB_OK|MB_ICONSTOP, "File '%s' can not be found. Your installation of CoolSetup is probably corrupted!", tgtplatform);
		load(projpath);
		SendMessage(progress, PBM_SETPOS, (WPARAM)0, 0);
		return 1;
	}
	if (file2==NULL) {
		fclose(file2);
		MsgBox(MB_OK|MB_ICONSTOP, "File %s can not be opened!", outputpath);
		load(projpath);
		SendMessage(progress, PBM_SETPOS, (WPARAM)0, 0);
		return 1;
	}
	for (;;) {
		i=fgetc(file);
		if (i==EOF)
			break;
		fputc(i, file2);
	}
	fclose(file);

	strcpy(buf, outputpath);
	strcat(buf, ".zip");
	file=fopen(buf, "rb");
	if (file==NULL) {
		if (file2!=NULL)
			fclose(file2);
		MsgBox(MB_OK|MB_ICONSTOP, "File '%s' can not be found. Your installation of CoolSetup is probably corrupted!", buf);
		load(projpath);
		SendMessage(progress, PBM_SETPOS, (WPARAM)0, 0);
		return 1;
	}
	for (;;) {
		i=fgetc(file);
		if (i==EOF)
			break;
		fputc(i, file2);
	}
	fclose(file);
	fclose(file2);
	DeleteFile(buf);
	SendMessage(progress, PBM_SETSTEP, (WPARAM)20, 0);
	SendMessage(progress, PBM_STEPIT, 0, 0);

	SendMessage(progress, PBM_SETSTEP, (WPARAM)5, 0);
	SendMessage(progress, PBM_STEPIT, 0, 0);
	load(projpath);
	SendMessage(progress, PBM_STEPIT, 0, 0);
	MsgBox(MB_OK|MB_ICONINFORMATION, "Compilation complete! Your setup is prepared for distributing...");
	SendMessage(progress, PBM_SETPOS, (WPARAM)0, 0);
	return 0;
}

int installfile::save(char *path)
{
	FILE *file;
	int i, j;
	
	file=fopen(path, "wb");
	if (file==NULL)
		return 1;
	fprintf(file, "CoolSetupProject 1");
	fprintf(file, "\n;Please don't change this file. It has specific format and if you broke it, program may be not able to read it successfully...");
	fprintf(file, "\n%s\n%s\n%s\n%s\n%s", outputpath, windowcaption, bigcaption, copyright, uninstkey);
	fprintf(file, "\n%s\n%s\n%s\n%s\n%s\n%s", defprogdir, defmenudir, readmefile, licensefile, execinst, execuninst);
	fprintf(file, "\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d", showbgwin, gofullscreen, showwelcome, showlicense, showreadme, showtarget, enableuninstallation, compressionstrength, overwriteprompt, progdircustom, menudircustom);
	fprintf(file, "\n%lu\n%lu", topcolor, bottomcolor);
	fprintf(file, "\n%d", ajc);
	fprintf(file, "\n;Now installtypes are gonna be defined...");
	fprintf(file, "\n%d\n%d\n%d", fgno, rgno, itno);
	for (i=0; i<fgno; i++) {
		fprintf(file, "\n%s\n%d", fg[i]->name, fg[i]->fgino);
		for (j=0; j<fg[i]->fgino; j++) {
			fprintf(file, "\n%s", fg[i]->fgi[j]->source);
			fprintf(file, "\n%s", fg[i]->fgi[j]->target);
			fprintf(file, "\n%d", fg[i]->fgi[j]->type);
			fprintf(file, "\n%d", fg[i]->fgi[j]->link);
			fprintf(file, "\n%s", fg[i]->fgi[j]->lname);
			fprintf(file, "\n%s", fg[i]->fgi[j]->ltargetdir);
			fprintf(file, "\n%s", fg[i]->fgi[j]->liconpath);
			fprintf(file, "\n%d", fg[i]->fgi[j]->liconnum);
			fprintf(file, "\n%s", fg[i]->fgi[j]->lwrkdir);
			fprintf(file, "\n%s", fg[i]->fgi[j]->ldescr);
		}
	}
	for (i=0; i<rgno; i++) {
		fprintf(file, "\n%s\n%d", rg[i]->name, rg[i]->rgino);
		for (j=0; j<rg[i]->rgino; j++) {
			fprintf(file, "\n%d", rg[i]->rgi[j]->root);
			fprintf(file, "\n%s", rg[i]->rgi[j]->path);
			fprintf(file, "\n%s", rg[i]->rgi[j]->item);
			fprintf(file, "\n%s", rg[i]->rgi[j]->value);
		}
	}
	for (i=0; i<itno; i++) {
		fprintf(file, "\n%s\n%d %d\n%d %d", it[i]->name, it[i]->fgno, it[i]->fgcust, it[i]->rgno, it[i]->rgcust);
		for (j=0; j<it[i]->fgno; j++) {
			fprintf(file, "\n%d", it[i]->fg[j]);
		}
		for (j=0; j<it[i]->rgno; j++) {
			fprintf(file, "\n%d", it[i]->rg[j]);
		}
	}
	fclose(file);
	return 0;
}

int installfile::load(char *path)
{
	FILE *file;
	char buf[1024];
	int i, j, fno, rno, ino, ffgino, rrgino, ifgno, irgno;
	file=fopen(path, "rb");

	for (i=0; i<fgno; i++)
		delete fg[i];
	for (i=0; i<rgno; i++)
		delete rg[i];
	for (i=0; i<itno; i++)
		delete it[i];
	itno=fgno=rgno=0;
	for (i=0; i<32; i++) {
		fg[i]=NULL;
		rg[i]=NULL;
		it[i]=NULL;
	}

	curr=curf=curit=0;
	compressionstrength=6;
	ajc=1;

	if (file==NULL) {
		MessageBox(GetFocus(), projpath, "Can't open file", MB_OK);
		return 1;
	}

	ini_getnextline(file, buf);	
	if (!strcmp(buf, "CoolSetupProject 1")) {
		ini_getnextline(file, outputpath);
		ini_getnextline(file, windowcaption);
		ini_getnextline(file, bigcaption);
		ini_getnextline(file, copyright);
		ini_getnextline(file, uninstkey);
		ini_getnextline(file, defprogdir);
		ini_getnextline(file, defmenudir);
		ini_getnextline(file, readmefile);
		ini_getnextline(file, licensefile);
		ini_getnextline(file, execinst);
		ini_getnextline(file, execuninst);
		showbgwin=ini_getnextint(file);
		gofullscreen=ini_getnextint(file);
		showwelcome=ini_getnextint(file);
		showlicense=ini_getnextint(file);
		showreadme=ini_getnextint(file);
		showtarget=ini_getnextint(file);
		enableuninstallation=ini_getnextint(file);
		compressionstrength=ini_getnextint(file);
		overwriteprompt=ini_getnextint(file);
		progdircustom=ini_getnextint(file);
		menudircustom=ini_getnextint(file);
		topcolor=ini_getnextint(file);
		bottomcolor=ini_getnextint(file);
		ajc=ini_getnextint(file);
		
		fno=ini_getnextint(file);
		rno=ini_getnextint(file);
		ino=ini_getnextint(file);
		
		for (i=0; i<fno; i++) {
			ini_getnextline(file, buf);
			newfgrp(buf);
			if (fg[i]==NULL) {
				MsgBox(MB_OK|MB_ICONSTOP, "Out of memory!");
				fclose(file);
				return 1;
			}
			ffgino=ini_getnextint(file);
			
			for (j=0; j<ffgino; j++) {			
				if (fg[i]->additem()) {
					MsgBox(MB_OK|MB_ICONSTOP, "Out of memory!");
					fclose(file);
					return 1;
				}
				
				ini_getnextline(file, fg[i]->fgi[fg[i]->fgino-1]->source);
				ini_getnextline(file, fg[i]->fgi[fg[i]->fgino-1]->target);
				fg[i]->fgi[fg[i]->fgino-1]->type=ini_getnextint(file);
				fg[i]->fgi[fg[i]->fgino-1]->link=ini_getnextint(file);
				ini_getnextline(file, fg[i]->fgi[fg[i]->fgino-1]->lname);
				ini_getnextline(file, fg[i]->fgi[fg[i]->fgino-1]->ltargetdir);
				ini_getnextline(file, fg[i]->fgi[fg[i]->fgino-1]->liconpath);
				fg[i]->fgi[fg[i]->fgino-1]->liconnum=ini_getnextint(file);
				ini_getnextline(file, fg[i]->fgi[fg[i]->fgino-1]->lwrkdir);
				ini_getnextline(file, fg[i]->fgi[fg[i]->fgino-1]->ldescr);
			}
		}
		for (i=0; i<rno; i++) {
			ini_getnextline(file, buf);
			newrgrp(buf);
			if (rg[i]==NULL) {
				MsgBox(MB_OK|MB_ICONSTOP, "Out of memory!");
				fclose(file);
				return 1;
			}
			rrgino=ini_getnextint(file);
			
			for (j=0; j<rrgino; j++) {			
				if (rg[i]->additem()) {
					MsgBox(MB_OK|MB_ICONSTOP, "Out of memory!");
					fclose(file);
					return 1;
				}
				
				rg[i]->rgi[rg[i]->rgino-1]->root=ini_getnextint(file);
				ini_getnextline(file, rg[i]->rgi[rg[i]->rgino-1]->path);
				ini_getnextline(file, rg[i]->rgi[rg[i]->rgino-1]->item);
				ini_getnextline(file, rg[i]->rgi[rg[i]->rgino-1]->value);
			}
		}
		
		for (i=0; i<ino; i++) {
			ini_getnextline(file, buf);
			newitype(buf);
			if (it[i]==NULL) {
				MsgBox(MB_OK|MB_ICONSTOP, "Out of memory!");
				fclose(file);
				return 1;
			}
			ini_getnextline(file, buf);
			sscanf(buf, "%d %d", &ifgno, &it[i]->fgcust);
			//ifgno=ini_getnextint(file);
			ini_getnextline(file, buf);
			sscanf(buf, "%d %d", &irgno, &it[i]->rgcust);
			//irgno=ini_getnextint(file);
			
			for (j=0; j<ifgno; j++) {			
				if (it[i]->addfilegroup(ini_getnextint(file))) {
					MsgBox(MB_OK|MB_ICONSTOP, "Out of memory!");
					fclose(file);
					return 1;
				}
			}
			
			for (j=0; j<irgno; j++) {			
				if (it[i]->addreggroup(ini_getnextint(file))) {
					MsgBox(MB_OK|MB_ICONSTOP, "Out of memory!");
					fclose(file);
					return 1;
				}
			}
		}
	} else if (!strcmp(buf, "CoolSetupProject 2")) {
	} else {
		MessageBox(GetFocus(), projpath, "Corrupted file or usupported file format!", MB_OK);
		fclose(file);
		return 1;
	}
	fclose(file);
	return 0;
}