/*
    CoolSetup installation system
    Copyright (C) 2001-2002 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <unzip.h>
#include <stdlib.h>
#include <string.h>
#include <shlobj.h>
#include <time.h>
#include <sys\utime.h>
#include "../shared/instclass.h"
#include "./instclass.h"
#include "resource.h"

unzFile zipfile;
HINSTANCE hI;
installfile_installer instfile;
char *log;
char temppath[1024], setuppath[1024], curfp[1024], custmenudir[1024];
char buffer[65535];
char msgtitle[1024];
BITMAPINFO bmi;
BYTE *bits;
HBITMAP hbmp;
HDC memdc;
RECT rect;
HFONT font1, font2;
HWND hwnd, hwndprogress, hwndfile, hwnddlg;
int i, j, k, stopinstall, opr;
BROWSEINFO bi;
LPITEMIDLIST pidl;
LPITEMIDLIST pidlRoot;
HANDLE ithread;
FILE *uifile;

typedef struct __instlink {
	char *po, *pl,*desc, *ip, *wd, *arg;
	int in;
	__instlink *next;
} _instlink;

_instlink *firstlink, *curlink;

int StartDialogs(void);
int MsgBox(UINT fuStyle, char *pszFormat, ...);
int windows_registry_get_value(HKEY root, char *path, char *entry, int type1, void *datapointer, int sod);
HRESULT CreateLink(LPCSTR lpszPathObj, LPSTR lpszPathLink, LPSTR lpszDesc, LPSTR lpszIconPath, int nIconno, LPSTR lpszWorkDirectory, char *arg="") ;
BOOL APIENTRY UninstDlg(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam);
int unz_extract_file(unzFile zipfile, char *source, char *target, HWND parenthwnd);
int setup_execute(void);
BOOL CALLBACK WelcomeDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK ReadmeDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK LicenseDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK TypedestDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK CustomDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK OverwriteDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK CustomMenuDirDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
int ProceedInstall(void);
int InstallFile(filegroupitem *fgi, HWND parenthwnd);
int InstallRegItem(reggroupitem *rgi);

int WINAPI WinMain(HINSTANCE hIn, HINSTANCE, char *cmdline, int)
{
	_instlink *il, *nl;

	hI=hIn;

	strcpy(msgtitle, "Installation");
	InitCommonControls();

	if (strcmp(cmdline, "")) {
		strcpy(msgtitle, "Uninstallation");
		log=cmdline;
		DialogBox(hI, "UNINSTALL", NULL, (DLGPROC)UninstDlg);
		return 0;
	}
	CoInitialize(NULL);

	GetTempPath(1024, temppath);
	GetModuleFileName(hI, setuppath, 1024);
	zipfile=unzOpen(setuppath);
	unzMakeJumpTable(zipfile);
	if (zipfile==NULL) {
		MsgBox(MB_OK|MB_ICONSTOP, "Setup file is damaged!");
		return 0;
	}
	strcpy(buffer, "");
	sprintf(buffer, "%scstmp.ini", temppath);
	opr=0;
	unz_extract_file(zipfile, "coolsetup.cfg", buffer, NULL);
	strcpy(buffer, "");
	sprintf(buffer, "%scstmp.ini", temppath);
	instfile.load(buffer);
	DeleteFile(buffer);
	strcpy(msgtitle, instfile.windowcaption);
	firstlink=curlink=NULL;
	font1=CreateFont(30,23,0,0,FW_NORMAL,0,0,0,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Arial");
	font2=CreateFont(10,7,0,0,FW_NORMAL,0,0,0,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Arial");
	opr=instfile.overwriteprompt;

	setup_execute();
	unzClose(zipfile);
	CoUninitialize();
	il=firstlink;
	for (;;) {
		if (il==NULL)
			break;
		nl=il->next;
		GlobalFree(il->arg);
		GlobalFree(il->desc);
		GlobalFree(il->ip);
		GlobalFree(il->pl);
		GlobalFree(il->po);
		GlobalFree(il->wd);
		GlobalFree(il);
		il=nl;
	}
	return 0;
}

int csu_add(int type, char *path, char *item=NULL, int root=0, int rwk=0)
{
	if (!instfile.enableuninstallation)
		return 0;

	if (uifile==NULL)
		return 1;
	switch (type) {
	case 0:
		fprintf(uifile, "\nfile_created %s", path);
		break;
	case 1:
		fprintf(uifile, "\ndirectory_created %s", path);
		break;
	case 2:
		fprintf(uifile, "\nregentry_created %d %d %s\n:>%s", root, rwk, path, item);
		break;
	case 3:
		fprintf(uifile, "\nexecute_file %s", path);
		break;
	}
	return 0;
}

HRESULT CreateLink(LPCSTR lpszPathObj, LPSTR lpszPathLink, LPSTR lpszDesc, LPSTR lpszIconPath, int nIconno, LPSTR lpszWorkDirectory, char *arg)
{
	/*
	typedef struct {
	char *po, *pl,*desc, *ip, *wd, *arg;
	int in;
	} _instlink;
	
	_instlink *firstlink, *curlink;
	*/
	
	if (firstlink==NULL) {
		firstlink=(__instlink*)GlobalAlloc(GPTR, sizeof(_instlink));
		if (firstlink==NULL)
			return 1;
		firstlink->po=(char*)GlobalAlloc(GPTR, strlen(lpszPathObj)+1);
		strcpy(firstlink->po, lpszPathObj);
		firstlink->pl=(char*)GlobalAlloc(GPTR, strlen(lpszPathLink)+1);
		strcpy(firstlink->pl, lpszPathLink);
		firstlink->desc=(char*)GlobalAlloc(GPTR, strlen(lpszDesc)+1);
		strcpy(firstlink->desc, lpszDesc);
		firstlink->ip=(char*)GlobalAlloc(GPTR, strlen(lpszIconPath)+1);
		strcpy(firstlink->ip, lpszIconPath);
		firstlink->wd=(char*)GlobalAlloc(GPTR, strlen(lpszWorkDirectory)+1);
		strcpy(firstlink->wd, lpszWorkDirectory);
		firstlink->arg=(char*)GlobalAlloc(GPTR, strlen(arg)+1);
		strcpy(firstlink->arg, arg);
		firstlink->in=nIconno;
		firstlink->next=NULL;
		curlink=firstlink;
	} else {
		curlink->next=(__instlink*)GlobalAlloc(GPTR, sizeof(_instlink));
		if (curlink->next==NULL)
			return 1;
		curlink=curlink->next;
		curlink->po=(char*)GlobalAlloc(GPTR, strlen(lpszPathObj)+1);
		strcpy(curlink->po, lpszPathObj);
		curlink->pl=(char*)GlobalAlloc(GPTR, strlen(lpszPathLink)+1);
		strcpy(curlink->pl, lpszPathLink);
		curlink->desc=(char*)GlobalAlloc(GPTR, strlen(lpszDesc)+1);
		strcpy(curlink->desc, lpszDesc);
		curlink->ip=(char*)GlobalAlloc(GPTR, strlen(lpszIconPath)+1);
		strcpy(curlink->ip, lpszIconPath);
		curlink->wd=(char*)GlobalAlloc(GPTR, strlen(lpszWorkDirectory)+1);
		strcpy(curlink->wd, lpszWorkDirectory);
		curlink->arg=(char*)GlobalAlloc(GPTR, strlen(arg)+1);
		strcpy(curlink->arg, arg);
		curlink->in=nIconno;
		curlink->next=NULL;
	}
	return 0;
}

HRESULT CreateLink_true(LPCSTR lpszPathObj, LPSTR lpszPathLink, LPSTR lpszDesc, LPSTR lpszIconPath, int nIconno, LPSTR lpszWorkDirectory, char *arg)
{ 
    HRESULT hres; 
    IShellLink* psl; 

	//MsgBox(0, "pathobj: %s\npathling: %s\ndesc: %s\niconpath: %s\niconno: %d\nworkdir: %s\narg %s", lpszPathObj, lpszPathLink, lpszDesc, lpszIconPath, nIconno, lpszWorkDirectory, arg);
    hres = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (void**)&psl);
    if (SUCCEEDED(hres)) { 
        IPersistFile* ppf; 
		
        psl->SetPath(lpszPathObj); 
        psl->SetDescription(lpszDesc); 
		psl->SetIconLocation(lpszIconPath, nIconno);
		psl->SetWorkingDirectory(lpszWorkDirectory);
		if (strcmp(arg, ""))
			psl->SetArguments(arg);
        hres = psl->QueryInterface(IID_IPersistFile, (void**)&ppf); 
		
        if (SUCCEEDED(hres)) { 
            WORD wsz[MAX_PATH]; 
			
            MultiByteToWideChar(CP_ACP, 0, lpszPathLink, -1, wsz, MAX_PATH); 
			
            hres = ppf->Save(wsz, TRUE); 
            ppf->Release();
			if (hres==S_OK)
				csu_add(0, (char*)lpszPathLink, NULL, 0);
        } 
        psl->Release(); 
    }
	else
		MsgBox(MB_ICONWARNING|MB_OK, "Can't create shortcut '%s'!", lpszPathLink);

    return hres; 
} 

int windows_registry_get_value(HKEY root, char *path, char *entry, int type1, void *datapointer, int sod)
{
	HKEY key;
	DWORD size=sod, type;

	if (RegOpenKeyEx(root, path, 0, KEY_READ, &key) != ERROR_SUCCESS)
		return 1;
	RegQueryValueEx(key, entry, NULL, &type, (LPBYTE)datapointer, &size);
	RegCloseKey(key);
	if (type == REG_SZ && type1 == 0)
		return 0;
	if (type == REG_DWORD && type1 == 1)
		return 0;
	return 1;
}

int MsgBox(UINT fuStyle, char *pszFormat, ...)
{
    va_list     va;
    char		msgMessage[2048];
    int         n;

    va_start(va, pszFormat);
    vsprintf(msgMessage, pszFormat, va);
    va_end(va);

    n = MessageBox(GetFocus(), msgMessage, msgtitle, fuStyle|MB_SETFOREGROUND);

    return n;
}


int unz_extract_file(unzFile zipfile, char *source, char *target, HWND parenthwnd)
{
	FILE *file;
	int drc, tmp;
	unz_file_info ufi;
	struct _utimbuf ut;
	struct tm newdate;
	unz_file_info pfi;

	tmp=unzLocateFile(zipfile, source, 0);
	if (tmp!=UNZ_OK) {
		MsgBox(MB_OK|MB_ICONSTOP, "File was not found (%s)!", source);
		return 1;
	}
	
	if (opr!=0) {
		file=fopen(target, "rb");
		if (file!=NULL) {
			int x;
			
			fclose(file);

			if (opr==-1)
				return 0;
			strcpy(curfp, target);
			x=DialogBox(hI, "OVERWRITE", parenthwnd, (DLGPROC)OverwriteDlgProc);
			if (x==0)
				return 0;
			if (x==2)
				opr=0;
			if (x==-1) {
				opr=-1;
				return 0;
			}
		}
	}

	file=fopen(target, "wb");
	if (file==NULL)
		return 1;

	unzGetCurrentFileInfo(zipfile, &ufi, NULL, NULL, NULL, NULL, NULL, NULL);
	unzOpenCurrentFile(zipfile);
	for (;;) {
		drc=unzReadCurrentFile(zipfile, (void*)buffer, 65536);
		if (drc < 0) {
			MsgBox(MB_OK | MB_ICONSTOP, "Archive is corrupted! (item %s)", source);
			break;
		}
		if (drc <= 0)
			break;
		if (fwrite((void*)buffer, drc, 1, file) < 1) {
			MsgBox(MB_OK | MB_ICONSTOP, "Can't write to '%s'", target);
			break;
		}
	}
	unzCloseCurrentFile(zipfile);
	fclose(file);
	unzGetCurrentFileInfo(zipfile, &pfi, NULL, 0, NULL, 0, NULL, 0);
	newdate.tm_sec = pfi.tmu_date.tm_sec;
	newdate.tm_min=pfi.tmu_date.tm_min;
	newdate.tm_hour=pfi.tmu_date.tm_hour;
	newdate.tm_mday=pfi.tmu_date.tm_mday;
	newdate.tm_mon=pfi.tmu_date.tm_mon;
	if (pfi.tmu_date.tm_year > 1900)
		newdate.tm_year=pfi.tmu_date.tm_year - 1900;
	else
		newdate.tm_year=pfi.tmu_date.tm_year ;
	newdate.tm_isdst=-1;
	
	ut.actime=ut.modtime=mktime(&newdate);
	utime(target, (struct utimbuf*)&ut);
	
	return 0;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	switch (iMsg) {
	case WM_CREATE:
		SetTimer(hwnd, 1, 50, 0);
		return 0;
	case WM_ERASEBKGND:
		return 0;
	case WM_TIMER:
		KillTimer(hwnd, 1);
		StartDialogs();
		DestroyWindow(hwnd);
		return 0;
	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hdc;
			GetClientRect(hwnd, &rect);
			hdc=BeginPaint(hwnd, &ps);
			BitBlt(hdc, 0, 0, rect.right, rect.bottom, memdc, 0, 0, SRCCOPY);
			EndPaint(hwnd, &ps);
		}
		return 0;
	case WM_CLOSE:
		DestroyWindow(hwnd);
		return 0;
	case WM_DESTROY:
		DeleteDC(memdc);
		DeleteObject(hbmp);
		DeleteObject(font1);
		DeleteObject(font2);
		//GlobalFree(bits);
		PostQuitMessage(0);
		return 0;
	}
	return DefWindowProc(hwnd, iMsg, wParam, lParam);
}

int setup_execute(void)
{
	WNDCLASS wndclass;
	MSG msg;
	DWORD style;
	COLORREF tc, bc;
	HDC hwdc;
	RECT rect;
	/*TRIVERTEX        vert[2];
	GRADIENT_RECT    gRect;*/
	HBRUSH br;
	float v1, v2, v3, v4, v5, v6, v7, v8, v9;

	wndclass.style = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc = WndProc;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;
	wndclass.hInstance = hI;
	wndclass.hIcon = LoadIcon(hI, "MAINICON");
	wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground = NULL;
	wndclass.lpszMenuName = NULL;
	wndclass.lpszClassName = "CoolSetupInstance";
	RegisterClass(&wndclass);
	
	if (instfile.gofullscreen==0)
		style=WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_THICKFRAME|WS_MINIMIZEBOX;
	else
		style=WS_POPUP|WS_SYSMENU;

	hwnd=CreateWindow("CoolSetupInstance", instfile.windowcaption, style, 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN), NULL, NULL, hI, NULL);
	GetClientRect(hwnd, &rect);
	float xs=(float)(rect.right-rect.left), ys=(float)(rect.bottom-rect.top);

	//Associated variables init.
	bmi.bmiHeader.biWidth = (long)xs;
	bmi.bmiHeader.biHeight = (long)ys;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = 24;
	bmi.bmiHeader.biCompression = BI_RGB;
	bmi.bmiHeader.biSizeImage = 0;
	bmi.bmiHeader.biXPelsPerMeter = 0;
	bmi.bmiHeader.biYPelsPerMeter = 0;
	bmi.bmiHeader.biClrUsed = 0;
	bmi.bmiHeader.biClrImportant = 0;
	bmi.bmiHeader.biSize = sizeof(bmi.bmiHeader);
	
	//bits = (BYTE*)GlobalAlloc(GPTR, (unsigned long)(xs*ys*3));

	if ((GetRValue(instfile.topcolor)+GetGValue(instfile.topcolor)+GetBValue(instfile.topcolor))/3 > 128)
		tc=0;
	else
		tc=RGB(255, 255, 255);

	if ((GetRValue(instfile.bottomcolor)+GetGValue(instfile.bottomcolor)+GetBValue(instfile.bottomcolor))/3 > 128)
		bc=0;
	else
		bc=RGB(255, 255, 255);

	hwdc=GetDC(hwnd);
	memdc = CreateCompatibleDC(hwdc);
	hbmp=CreateDIBitmap(hwdc, &bmi.bmiHeader, 0, (void**)&bits, &bmi, DIB_RGB_COLORS);
	SelectObject(memdc, hbmp);
	
	v1=GetRValue(instfile.topcolor);
	v2=GetGValue(instfile.topcolor);
	v3=GetBValue(instfile.topcolor);
	
	v4=GetRValue(instfile.bottomcolor);
	v5=GetGValue(instfile.bottomcolor);
	v6=GetBValue(instfile.bottomcolor);
	
	v7=(v4-v1)/ys;
	v8=(v5-v2)/ys;
	v9=(v6-v3)/ys;
	
	for (i=0; i<ys; i++) {
		br=CreateSolidBrush(RGB((BYTE)v1, (BYTE)v2, (BYTE)v3));
		rect.top=i;
		rect.bottom=i+1;
		rect.left=0;
		rect.right=(long)xs;
		FillRect(memdc, &rect, br);
		DeleteObject(br);
		v1+=v7;
		v2+=v8;
		v3+=v9;
	}
	
 
	SetBkMode(memdc, TRANSPARENT);
	SelectObject(memdc, font1);
	SetTextColor(memdc, tc);
	TextOut(memdc, 2, 2, instfile.bigcaption, strlen(instfile.bigcaption));
	SelectObject(memdc, font2);
	SetTextColor(memdc, bc);
	TextOut(memdc, rect.right-7*strlen(instfile.copyright)-5, rect.bottom-15, instfile.copyright, strlen(instfile.copyright));

	if (instfile.showbgwin!=0)
		ShowWindow(hwnd, SW_SHOW);
	else
		ShowWindow(hwnd, SW_SHOWMINIMIZED);
	UpdateWindow(hwnd);
	ReleaseDC(hwnd, hwdc);
	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return 0;
}

int mymkdir(char *buffer)
{
	int a = CreateDirectory(buffer, NULL);
	if ((a == 0) && (GetLastError() == 183))
		return 0;
	if (a == 0)
		return 1;
	csu_add(1, buffer, NULL, 0);
	//RegistryAdd(1, buffer);
	return 0;
}

int CreateDir(char *newdir)
{
	char *buffer;
	char *p;
	int  len = strlen(newdir);  
	
	if (len <= 0) 
		return 0;
	
	buffer = (char*)malloc(len+1);
	strcpy(buffer,newdir);
	
	if (buffer[len-1] == '/') {
		buffer[len-1] = '\0';
	}
	if (mymkdir(buffer) == 0) {
		free(buffer);
		return 0;
    }
	
	p = buffer+3;
	while (1) {
		char hold;
		
		while(*p && *p != '\\' && *p != '/')
			p++;
		hold = *p;
		*p = 0;
		if ((mymkdir(buffer) != 0)) {
			MsgBox(MB_OK, "Couldn't create directory '%s'!", buffer);
			free(buffer);
			return 1;
        }
		if (hold == 0)
			break;
		*p++ = hold;
    }
	free(buffer);
	return 0;
}

int unzipGetFile(char *filename)
{
	int tmp;
	unz_file_info ufi;

	tmp=unzLocateFile(zipfile, filename, 0);
	if (tmp!=UNZ_OK) {
//		MessageBox(GetFocus(), filename, "File was not found:", MB_OK | MB_ICONSTOP);
		return 1;
	}
	unzGetCurrentFileInfo(zipfile, &ufi, NULL, NULL, NULL, NULL, NULL, NULL);
	if (ufi.uncompressed_size >= 65534) {
//		MessageBox(GetFocus(), filename, "File is too long:", MB_OK | MB_ICONSTOP);
		return 2;
	}
	unzOpenCurrentFile(zipfile);
	if (unzReadCurrentFile(zipfile, (void*)buffer, ufi.uncompressed_size) <0) {
//		MessageBox(GetFocus(), filename, "File can't be read:", MB_OK | MB_ICONSTOP);
		return 3;
	}
	buffer[ufi.uncompressed_size]='\0';
	unzCloseCurrentFile(zipfile);
	return 0;
}

int StartDialogs(void)
{
	struct {char *name; DLGPROC dp;} dlgs[10];
	int i=0, count, rv;

	if (instfile.showwelcome) {
		dlgs[i].name="WELCOME";
		dlgs[i].dp=WelcomeDlgProc;
		i++;
	}
	if (instfile.showlicense) {
		dlgs[i].name="LICENSE";
		dlgs[i].dp=LicenseDlgProc;
		i++;
	}
	if (instfile.showreadme) {
		dlgs[i].name="README";
		dlgs[i].dp=ReadmeDlgProc;
		i++;
	}
	if (instfile.showtarget) {
		dlgs[i].name="TYPEDEST";
		dlgs[i].dp=TypedestDlgProc;
		i++;
	}
	if (instfile.menudircustom) {
		dlgs[i].name="CMENUDIR";
		dlgs[i].dp=CustomMenuDirDlgProc;
		i++;
	}
	count=i;
	i=0;

back1:
	for (j=0; j<instfile.it[instfile.selit]->fgno; j++)
		instfile.fg[instfile.it[instfile.selit]->fg[j]]->installme=0;

	for (;;) {
		rv=DialogBox(hI, dlgs[i].name, hwnd, dlgs[i].dp);
		if (rv==0)
			return 0;
		if (rv==-1&&i>0)
			i--;
		if (rv==1)
			i++;
		if (i>=count)
			break;
	}

//back2:
	if (instfile.it[instfile.selit]->fgcust) {
		rv=DialogBox(hI, "CUSTOM", hwnd, CustomDlgProc);
		if (rv==0)
			return 0;
		if (rv==-1) {
			i--;
			goto back1;
		}
	}
	else {
		for (i=0; i<instfile.it[instfile.selit]->fgno; i++)
			instfile.fg[instfile.it[instfile.selit]->fg[i]]->installme=1;
	}

	for (j=0; j<instfile.it[instfile.selit]->rgno; j++)
		instfile.rg[instfile.it[instfile.selit]->rg[j]]->installme=1;

	strcpy(instfile.defmenudir, "%windowsmenuprograms%\\");
	strcat(instfile.defmenudir, custmenudir);
	instfile.parse_path(instfile.defmenudir);
	//Now all needed data are accesable, so we can begin installation ;-)
	ProceedInstall();
	/*if (instfile.it[instfile.selit]->rgcust) {
		rv=DialogBox(hI, dlgs[i].name, hwnd, dlgs[i].dp);
		if (rv==0)
			return 0;
		if (rv==-1) {
			i--;
			goto back2;
		}
	}*/
	return 0;
}

BOOL CALLBACK WelcomeDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		strcpy(buffer, "");
		sprintf(buffer, "Welcome to %s setup program. This will install this program on your computer.", instfile.bigcaption);
		SetDlgItemText(hDlg, IDC_STATIC1, buffer);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_NEXT:
			EndDialog(hDlg, 1);
			return 0;
		case ID_EXIT:
			if (MsgBox(MB_YESNO|MB_ICONWARNING, "Do you really want to exit this installation?")==IDNO)
				return 0;
			EndDialog(hDlg, 0);
			return 0;
		case ID_PREV:
			EndDialog(hDlg, -1);
			return 0;
		}
	}
	return 0;
}

BOOL CALLBACK ReadmeDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		unzipGetFile("readme.csf");
		SetDlgItemText(hDlg, IDC_EDIT1, buffer);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_NEXT:
			EndDialog(hDlg, 1);
			return 0;
		case ID_EXIT:
			if (MsgBox(MB_YESNO|MB_ICONWARNING, "Do you really want to exit this installation?")==IDNO)
				return 0;
			EndDialog(hDlg, 0);
			return 0;
		case ID_PREV:
			EndDialog(hDlg, -1);
			return 0;
		}
	}
	return 0;
}

BOOL CALLBACK LicenseDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		unzipGetFile("license.csf");
		SetDlgItemText(hDlg, IDC_EDIT1, buffer);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_NEXT:
			EndDialog(hDlg, 1);
			return 0;
		case ID_EXIT:
			if (MsgBox(MB_YESNO|MB_ICONWARNING, "If you don't accept this license, you can't continue installation. Do you really want to exit?")==IDNO)
				return 0;
			EndDialog(hDlg, 0);
			return 0;
		case ID_PREV:
			EndDialog(hDlg, -1);
			return 0;
		}
	}
	return 0;
}

BOOL CALLBACK TypedestDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		SetDlgItemText(hDlg, IDC_EDIT1, instfile.defprogdir);
		if (!instfile.progdircustom) {
			EnableWindow(GetDlgItem(hDlg, IDC_EDIT1), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_BUTTON1), FALSE);
		}
		for (i=0; i<instfile.itno; i++)
			SendDlgItemMessage(hDlg, IDC_LIST1, LB_ADDSTRING, 0, (LPARAM) instfile.it[i]->name);
		SendDlgItemMessage(hDlg, IDC_LIST1, LB_SETCURSEL, (WPARAM) 0, (LPARAM) 0);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_NEXT:
			GetDlgItemText(hDlg, IDC_EDIT1, instfile.defprogdir, 512);
			instfile.selit = SendDlgItemMessage(hDlg, IDC_LIST1, LB_GETCURSEL, (WPARAM) 0, (LPARAM) 0);
			EndDialog(hDlg, 1);
			return 0;
		case ID_EXIT:
			if (MsgBox(MB_YESNO|MB_ICONWARNING, "Do you really want to exit this installation?")==IDNO)
				return 0;
			EndDialog(hDlg, 0);
			return 0;
		case ID_PREV:
			EndDialog(hDlg, -1);
			return 0;
		case IDC_BUTTON1:
				GetDlgItemText(hDlg, IDC_EDIT1, instfile.defprogdir, 512);
				bi.hwndOwner=hDlg;
				bi.pidlRoot=NULL;
				bi.pszDisplayName=instfile.defprogdir;
				bi.lpszTitle="Please select program's target directory:";
				bi.ulFlags=BIF_RETURNONLYFSDIRS;
				bi.lpfn=NULL;
				bi.lParam=0L;
				bi.iImage=0;
				if (0 != SHGetSpecialFolderLocation(HWND_DESKTOP, CSIDL_DRIVES, &pidlRoot))
					return 0;
				if (NULL == pidlRoot)
					return 0;
				bi.pidlRoot = pidlRoot;
				pidl=SHBrowseForFolder(&bi);
				if (NULL != pidl)
					SHGetPathFromIDList(pidl, instfile.defprogdir);
				SetDlgItemText(hDlg, IDC_EDIT1, instfile.defprogdir);
			return 0;
		}
	}
	return 0;
}

BOOL CALLBACK CustomMenuDirDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	WIN32_FIND_DATA wfd;
	HANDLE ff;

	switch (uMsg) {
	case WM_INITDIALOG: 
		strcpy(buffer, "%windowsmenuprograms%\\");
		strcat(buffer, "*.*");
		instfile.parse_path(buffer);
		
		ff=FindFirstFile(buffer, &wfd);
		if (INVALID_HANDLE_VALUE==ff)
			return TRUE;
		if (wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY && strcmp(wfd.cFileName, ".") && strcmp(wfd.cFileName, ".."))
			SendDlgItemMessage(hDlg, IDC_COMBO1, CB_INSERTSTRING, (WPARAM)-1, (LPARAM) wfd.cFileName);
		for (;;) {
			if (FindNextFile(ff, &wfd)!=0) {
				if (wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY && strcmp(wfd.cFileName, ".") && strcmp(wfd.cFileName, ".."))
					SendDlgItemMessage(hDlg, IDC_COMBO1, CB_INSERTSTRING, (WPARAM)-1, (LPARAM) wfd.cFileName);
			}
			else
				break;
		}
		FindClose(ff);
		SendDlgItemMessage(hDlg, IDC_COMBO1, WM_SETTEXT, (WPARAM)0, (LPARAM)instfile.defmenudir);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_NEXT:
			SendDlgItemMessage(hDlg, IDC_COMBO1, WM_GETTEXT, (WPARAM)_MAX_PATH, (LPARAM)custmenudir);
			EndDialog(hDlg, 1);
			return 0;
		case ID_EXIT:
			if (MsgBox(MB_YESNO|MB_ICONWARNING, "Do you really want to exit this installation?")==IDNO)
				return 0;
			EndDialog(hDlg, 0);
			return 0;
		case ID_PREV:
			EndDialog(hDlg, -1);
			return 0;
		}
	}
	return 0;
}

BOOL CALLBACK CustomDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		for (i=0; i<instfile.it[instfile.selit]->fgno; i++)
			SendDlgItemMessage(hDlg, IDC_LIST1, LB_ADDSTRING, 0, (LPARAM) instfile.fg[instfile.it[instfile.selit]->fg[i]]->name);
		SendDlgItemMessage(hDlg, IDC_LIST1, LB_SETSEL, (WPARAM)TRUE, (LPARAM)-1);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_NEXT:
			for (i=0; i<instfile.it[instfile.selit]->fgno; i++)
				instfile.fg[instfile.it[instfile.selit]->fg[i]]->installme = SendDlgItemMessage(hDlg, IDC_LIST1, LB_GETSEL, (WPARAM)i, 0);
			EndDialog(hDlg, 1);
			return 0;
		case ID_EXIT:
			if (MsgBox(MB_YESNO|MB_ICONWARNING, "Do you really want to exit this installation?")==IDNO)
				return 0;
			EndDialog(hDlg, 0);
			return 0;
		case ID_PREV:
			EndDialog(hDlg, -1);
			return 0;
		}
	}
	return 0;
}

BOOL CALLBACK OverwriteDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	char buffer[1024];
	DWORD fa;
	HANDLE file2;
	FILETIME ft1, ft2, ft3;
	SYSTEMTIME st2;
	unz_file_info pfi;

	switch(uMsg) {
		case WM_INITDIALOG:
			SetDlgItemText(hDlg, IDC_STATIC1, curfp);
			fa=GetFileAttributes(curfp);
			file2=CreateFile(curfp, 0, 0, NULL, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, NULL);
			GetFileTime(file2, &ft1, &ft2, &ft3);
			CloseHandle(file2);
			FileTimeToLocalFileTime(&ft3, &ft1);
			FileTimeToSystemTime(&ft1, &st2);
			strcpy(buffer, "");
			sprintf(buffer, "%c%c%c%c\n%d.%d %d (%02d:%02d.%02d)", fa&FILE_ATTRIBUTE_ARCHIVE?'A':'-', fa&FILE_ATTRIBUTE_HIDDEN?'H':'-', fa&FILE_ATTRIBUTE_SYSTEM?'S':'-', fa&FILE_ATTRIBUTE_READONLY?'R':'-', st2.wDay, st2.wMonth, st2.wYear, st2.wHour, st2.wMinute, st2.wSecond);
			SetDlgItemText(hDlg, IDC_STATIC2, buffer);
			unzGetCurrentFileInfo(zipfile, &pfi, NULL, 0, NULL, 0, NULL, 0);
			strcpy(buffer, "");
			sprintf(buffer, "A---\n%u.%u %u (%02u:%02u.%02u)", pfi.tmu_date.tm_mday, pfi.tmu_date.tm_mon+1, pfi.tmu_date.tm_year, pfi.tmu_date.tm_hour, pfi.tmu_date.tm_min, pfi.tmu_date.tm_sec);
			SetDlgItemText(hDlg, IDC_STATIC5, buffer);
			return TRUE;
		case WM_COMMAND:
			switch (LOWORD(wParam)) {
			case IDNO:
				EndDialog(hDlg, 0);
				return 0;
			case IDYES:
				EndDialog(hDlg, 1);
				return 0;
			case IDOWALL:
				EndDialog(hDlg, 2);
				return 0;
			case IDOWNONE:
				EndDialog(hDlg, -1);
				return 0;
			}
			break;
	}
	return 0;
}

BOOL CALLBACK InstallationDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	long c=0;
	_instlink *cl;

	switch (uMsg) {
	case WM_INITDIALOG:
		hwndprogress=GetDlgItem(hDlg, IDC_PROGRESS1);
		hwndfile=GetDlgItem(hDlg, IDC_CF);
		hwnddlg=hDlg;
		for (i=0; i<instfile.fgno; i++) {
			if (instfile.fg[i]->installme)
				c+=instfile.fg[i]->fgino;
		}
		for (i=0; i<instfile.rgno; i++) {
			if (instfile.rg[i]->installme)
				c+=instfile.rg[i]->rgino;
		}
		c++;
		SendMessage(hwndprogress, PBM_SETRANGE, 0, MAKELPARAM(0, c));
		SendMessage(hwndprogress, PBM_SETSTEP, (WPARAM)1, 0);
		stopinstall=0;
		ResumeThread(ithread);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case 1000: //Make links
			cl=firstlink;
			for (;;) {
				if (cl==NULL)
					break;
				CreateLink_true(cl->po, cl->pl, cl->desc, cl->ip, cl->in, cl->wd, cl->arg);
				cl=cl->next;
			}
			SendMessage(hwndprogress, PBM_STEPIT, 0, 0);
			return 0;
		case IDCANCEL:
			if (stopinstall!=2)
				stopinstall=1;
			return 0;
		case IDOK:
			MsgBox(MB_OK, "Installation complete!");
			EndDialog(hDlg, 0);
			return 0;
		}
	}
	return 0;
}

DWORD WINAPI InstallationFunction(LPVOID lpParameter)
{
	int cfg=0, cfgi=0, crg=0, crgi=0;
	uifile=NULL;
	HKEY hKey;
	DWORD disp;
	char buf2[1024];
	char path[2048];

	if (instfile.enableuninstallation) {
		GetWindowsDirectory(path, 2048);
		strcat(path, "\\Application data\\");
		strcat(path, instfile.uninstkey);
		strcat(path, ".csu");
		uifile=fopen(path, "rb");
		if (uifile!=NULL) {
			fclose(uifile);
			if (MessageBox(GetFocus(), "There is previous or current version of this program already installed. Would you like to uninstall it first? If not, you are risking, that next uninstallation may not work correctly!", "Warning", MB_YESNO | MB_ICONWARNING)==IDYES) {
				log=path;
				DialogBox(hI, "UNINSTALL", NULL, (DLGPROC)UninstDlg);
			}
		}
		uifile=fopen(path, "wb");
		if (uifile==NULL)
			MsgBox(MB_OK|MB_ICONWARNING, "There was an error during creation of '%s' uninstall log. Uninstallation will be not available!", path);
		if (uifile)
			fprintf(uifile, "CoolSetup_uninstallation_log version 1");

	}
	if (instfile.execuninst[0]!='\0') {
		instfile.parse_path(instfile.execuninst);
		csu_add(3, instfile.execuninst);
	}
	for (;;) {
		if (stopinstall==1)
			break;

		if (cfg!=-1) {
			if (cfg>=instfile.fgno) {
				cfg=-1;
				continue;
			}
			if (instfile.fg[cfg]->installme) {
				if (cfgi<instfile.fg[cfg]->fgino) {
					InstallFile(instfile.fg[cfg]->fgi[cfgi], hwnddlg);
					cfgi++;
				}
				else {
					cfg++;
					cfgi=0;
				}
			}
			else
				cfg++;
			
			if (cfg<instfile.fgno)
				continue;
			else
				cfg=-1;
		}

		else if (crg!=-1) {
			if (crg>=instfile.rgno) {
				crg=-1;
				continue;
			}
			if (instfile.rg[crg]->installme) {
				if (crgi<instfile.rg[crg]->rgino) {
					InstallRegItem(instfile.rg[crg]->rgi[crgi]);
					crgi++;
				}
				else {
					crg++;
					crgi=0;
				}
			}
			else
				crg++;

			if (crg<instfile.rgno)
				continue;
			else
				crg=-1;
		}
		else break;
	}

	if (instfile.enableuninstallation) {
		FILE *in, *out;
		int c;

		GetWindowsDirectory(buf2, 256);
		strcat(buf2, "\\");
		strcat(buf2, "csuninst.exe");
		j=unzGetZipfOffset(zipfile);
		out=fopen(buf2, "wb");
		if (out==NULL) {
			MsgBox(MB_OK|MB_ICONSTOP, "Can't copy uninstaller '%s'! Uninstall will be not possible, sorry...", buf2);
			goto next;
		}

		in=fopen(setuppath, "rb");
		if (in==NULL) {
			fclose(out);
			MsgBox(MB_OK|MB_ICONSTOP, "Can't open installer file '%s'! Uninstall will be not possible, sorry...", setuppath);
			goto next;
		}

		for (i=0; i<j; i++) {
			c=fgetc(in);
			fputc(c, out);
		}
		fclose(in);
		fclose(out);

		strcpy(buffer, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\");
		strcat(buffer, instfile.uninstkey);

		if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, buffer, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, &disp) == ERROR_SUCCESS) {
			csu_add(2, buffer, "", 2, 1);
			csu_add(2, buffer, "DisplayName", 2, 0);
			csu_add(2, buffer, "UninstallString", 2, 0);
			RegSetValueEx(hKey, "DisplayName", 0, REG_SZ, (unsigned const char*)instfile.bigcaption, strlen(instfile.bigcaption));
			GetWindowsDirectory(buf2, 256);
			strcat(buf2, "\\");
			strcat(buf2, "csuninst.exe");
			strcpy(buffer, buf2);
			strcat(buffer, " ");
			strcat(buffer, path);
			RegSetValueEx(hKey, "UninstallString", 0, REG_SZ, (unsigned const char*)buffer, strlen(buffer));
			//RegSetValueEx(hKey, "ExecuteBeforeUninstParams", 0, REG_SZ, (unsigned const char*)instfile.execinst, strlen(instfile.execinst));

			RegCloseKey(hKey);
		}
		
		strcpy(buf2, "%windowsdir%\\csuninst.exe");
		instfile.parse_path(buf2);
		
		if (strcmp(instfile.defmenudir, "")) {
			strcpy(buffer, "%targetmenudir%\\Uninstall application.lnk");
			instfile.parse_path(buffer);
			CreateLink(buf2, buffer, "This will uninstall this application...", buf2, 1, "", path);
			csu_add(0, buffer);
		}
	}
next:
	SendMessage(hwnddlg, WM_COMMAND, MAKEWPARAM(1000, 0), 0);
	if (strcmp(instfile.execinst, "")) {
		instfile.parse_path(instfile.execinst);
		ShellExecute(hwnddlg, "open", instfile.execinst, "", "", SW_SHOW);
	}
	stopinstall=2;
	if (uifile)
		fclose(uifile);
	SendMessage(hwnddlg, WM_COMMAND, MAKEWPARAM(IDOK, 0), 0);
	return 0;
}

int ProceedInstall(void)
{
	DWORD ti;
	ithread=CreateThread(NULL, 0, InstallationFunction, NULL, CREATE_SUSPENDED, &ti);
	DialogBox(hI, "INSTALLATION", hwnd, InstallationDlgProc);
	CloseHandle(ithread);
	return 0;
}

int InstallFile(filegroupitem *fgi, HWND parenthwnd)
{
	char target[_MAX_PATH], dir[_MAX_PATH], buffer[2048];
	int sod;

	strcpy(target, fgi->target);
	instfile.parse_path(target);
	SetWindowText(hwndfile, target);
	
	strcpy(dir, target);
	for (sod=strlen(dir); sod>=0; sod--) {
		if (dir[sod]=='\\'||dir[sod]=='/') {
			dir[sod]='\0';
			break;
		}
	}

	if (!SetCurrentDirectory(dir)) {
		if (CreateDir(dir))
			return -1;
		if (!SetCurrentDirectory(dir)) {
			MsgBox(MB_ICONSTOP|MB_OK, "Can't get in to directory: '%s'", dir);
			return -1;
		}
	}
	if (!strcmp(fgi->source, "%phatom_file%")||!strcmp(fgi->source, "%remove_file%")||!strcmp(fgi->source, "%free_file%")) {
		if (!strcmp(fgi->source, "%phatom_file%")) {
			csu_add(0, target, NULL, 0);
		} else if (!strcmp(fgi->source, "%remove_file%")) {
			DeleteFile(target);
		} else if (!strcmp(fgi->source, "%free_file%")) {
			FILE *f;
			f=fopen(target, "wb");
			fclose(f);
		}
	}
	else {
		if (unz_extract_file(zipfile, fgi->source, target, parenthwnd))
			MsgBox(MB_OK, "File can't be extracted: '%s'", target);
		csu_add(0, target, NULL, 0);
	}
	
	if (fgi->link) {
		char buf1[2028], buf2[2028];
		strcpy(dir, fgi->ltargetdir);
		instfile.parse_path(dir);

		strcpy(buffer, dir);
		strcat(buffer, "\\");
		strcat(buffer, fgi->lname);
		strcat(buffer, ".lnk");

		if (!SetCurrentDirectory(dir)) 	{
			if (CreateDir(dir))
				return -1;
			if (!SetCurrentDirectory(dir)) {
				MsgBox(MB_ICONSTOP|MB_OK, "Can't get in to directory: '%s'", dir);
				return -1;
			}
		}
		strcpy(buf1, fgi->lwrkdir);
		instfile.parse_path(buf1);
		strcpy(buf2, fgi->liconpath);
		instfile.parse_path(buf2);
		CreateLink(target, buffer, fgi->ldescr, buf2, fgi->liconnum, buf1);
	}
	SendMessage(hwndprogress, PBM_STEPIT, 0, 0);
	return 0;
}

int removeregkey(HKEY root, char *path, char *key, int wholekey);

int InstallRegItem(reggroupitem *rgi)
{
	HKEY rk, hKey;
	DWORD disp;
	char buf[2048];

	if (rgi->root==0)
		rk=HKEY_CLASSES_ROOT;
	if (rgi->root==1)
		rk=HKEY_CURRENT_USER;
	if (rgi->root==2)
		rk=HKEY_LOCAL_MACHINE;
	if (rgi->root==3)
		rk=HKEY_USERS;
	if (rgi->root==4)
		rk=HKEY_CURRENT_CONFIG;
	if (rgi->root==5)
		rk=HKEY_DYN_DATA;

	if (!strcmpi(rgi->value, "%phantom_regentry%")) {
		csu_add(2, rgi->path, rgi->item, rgi->root, 1);
	} else if (!strcmpi(rgi->value, "%remove_regentry%")) {
		removeregkey(rk, rgi->path, rgi->item, 1);
	} else {
		strcpy(buf, rgi->value);
		instfile.parse_path(buf);
		
		if (RegCreateKeyEx(rk, rgi->path, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, &disp) == ERROR_SUCCESS) {
			RegSetValueEx(hKey, rgi->item, 0, REG_SZ, (unsigned const char*)buf, strlen(buf));
			RegCloseKey(hKey);
		}
		csu_add(2, rgi->path, rgi->item, rgi->root, disp==REG_CREATED_NEW_KEY);
	}

	return 0;
}