/*
    CoolSetup installation system
    Copyright (C) 2001-2002 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <unzip.h>
#include <stdlib.h>
#include <string.h>
#include <shlobj.h>
#include <time.h>
#include <sys\utime.h>
#include "../shared/instclass.h"
#include "resource.h"

extern char* log;
HANDLE uithread=NULL;

class unifile {
public:
	~unifile() {if (path) delete []path;}
	char *path;
	unifile *next;
	unifile *prev;
};

class unidir {
public:
	~unidir() {if (path) delete []path;}
	char *path;
	unidir *next;
	unidir *prev;
};

class uniregi {
public:
	~uniregi() {if (path) delete []path; if (item) delete []item;}
	int root;
	int rwk;
	char *path;
	char *item;
	uniregi *next;
	uniregi *prev;
};

unifile *uifile1=NULL, *uifilen=NULL, *uifilee=NULL;
unidir *uidir1=NULL, *uidirn=NULL, *uidire=NULL;
uniregi *uiregi1=NULL, *uiregin=NULL, *uiregie=NULL;

int removeregkey(HKEY root, char *path, char *key, int wholekey)
{
	HKEY hk;

	if (wholekey==0) {
		if (RegOpenKeyEx(root, path, 0, KEY_ALL_ACCESS, &hk) != ERROR_SUCCESS)
			return 0;
		if (RegDeleteValue(hk, key)==ERROR_SUCCESS) {
			RegCloseKey(hk);
			return 0;
		}
		RegCloseKey(hk);
		return 1;
	} else {
		if (RegDeleteKey(root, path)!=ERROR_SUCCESS)
			return 1;
	}
	return 0;
}

int uninst_read_line(FILE *file, char *buffer)
{
	int i=0, c;
	for (;;) {
		c=fgetc(file);
		if (c==EOF||c==0x0a)
			break;
		buffer[i++]=c;
	}
	buffer[i]='\0';
	return c==EOF;
}

DWORD WINAPI uninstall_program(LPVOID parameter)
{
	HWND hDlg=(HWND)parameter;
	FILE *file;
	char buffer[_MAX_PATH], buf[2048];
	unifile *uf, *ufn;
	unidir *ud, *udn;
	uniregi *ur, *urn;
	int rr=0, rf=0, rd=0;
	char *re[6]={"HKCR", "HKCU", "HKLM", "HKU", "HKCC", "HKDD"};

	file=fopen(log, "rb");
	if (file==NULL) {
		SendDlgItemMessage(hDlg, IDC_LIST1, LB_ADDSTRING, 0, (LPARAM)"Can't read uninstallation log!");
		EnableWindow(GetDlgItem(hDlg, IDCANCEL), TRUE);
		return 0;
	}
	for (;;) {
		if (uninst_read_line(file, buffer))
			break;
		if (!strnicmp(buffer, "execute_file", strlen("execute_file"))) {
			ShellExecute(hDlg, "open", &buffer[13], NULL, "", SW_SHOW);
		}
		if (!strnicmp(buffer, "file_created", strlen("file_created"))) {
			if (uifile1==NULL) {
				uifile1=new unifile;
				uifile1->next=NULL;
				uifile1->path=new char[strlen(&buffer[13])];
				strcpy(uifile1->path, &buffer[13]);
				uifilen=uifile1;
				uifilee=uifile1;
				uifilen->prev=NULL;
			} else {
				uifilen->next=new unifile;
				uifilen->next->prev=uifilen;
				uifilen=uifilen->next;
				uifilen->next=NULL;
				uifilen->path=new char[strlen(&buffer[13])];
				uifilee=uifilen;
				strcpy(uifilen->path, &buffer[13]);
			}
		} else if (!strnicmp(buffer, "directory_created", strlen("directory_created"))) {
			if (uidir1==NULL) {
				uidir1=new unidir;
				uidir1->next=NULL;
				uidir1->path=new char[strlen(&buffer[18])];
				strcpy(uidir1->path, &buffer[18]);
				uidirn=uidir1;
				uidire=uidir1;
				uidirn->prev=NULL;
			} else {
				uidirn->next=new unidir;
				uidirn->next->prev=uidirn;
				uidirn=uidirn->next;
				uidirn->next=NULL;
				uidirn->path=new char[strlen(&buffer[18])];
				uidire=uidirn;
				strcpy(uidirn->path, &buffer[18]);
			}
		} else if (!strnicmp(buffer, "regentry_created", strlen("regentry_created"))) {
			if (uiregi1==NULL) {
				uiregi1=new uniregi;
				uiregi1->next=NULL;
				sscanf(&buffer[17], "%d %d", &uiregi1->root, &uiregi1->rwk);
				uiregi1->path=new char[strlen(&buffer[21])];
				strcpy(uiregi1->path, &buffer[21]);
				uiregin=uiregi1;
				uiregie=uiregi1;
				uiregin->prev=NULL;
			} else {
				uiregin->next=new uniregi;
				uiregie->next->prev=uiregin;
				uiregin=uiregin->next;
				uiregin->next=NULL;
				sscanf(&buffer[17], "%d %d", &uiregin->root, &uiregin->rwk);
				uiregin->path=new char[strlen(&buffer[21])];
				uiregie=uiregin;
				strcpy(uiregin->path, &buffer[21]);
			}
			if (uninst_read_line(file, buffer))
				break;
			uiregin->item=new char[strlen(&buffer[2])];
			strcpy(uiregin->item, &buffer[2]);
		}
	}
	fclose(file);
	
	ur=uiregie;
	for (;;) {
		HKEY hk;
		if (ur==NULL)
			break;
		if (ur->root==0)
			hk=HKEY_CLASSES_ROOT;
		if (ur->root==1)
			hk=HKEY_CURRENT_USER;
		if (ur->root==2)
			hk=HKEY_LOCAL_MACHINE;
		if (ur->root==3)
			hk=HKEY_USERS;
		if (ur->root==4)
			hk=HKEY_CURRENT_CONFIG;
		if (ur->root==5)
			hk=HKEY_DYN_DATA;

		if (removeregkey(hk, ur->path, ur->item, ur->rwk)) {
			if (rr==0) {
				SendDlgItemMessage(hDlg, IDC_LIST1, LB_ADDSTRING, 0, (LPARAM)"***Following registry entries weren't removed!***");
				rr=1;
			}
			sprintf(buf, "%s\\%s", re[ur->root], ur->path);
			SendDlgItemMessage(hDlg, IDC_LIST1, LB_ADDSTRING, 0, (LPARAM)buf);
		}
		urn=ur->prev;
		delete ur;
		ur=urn;
	}
	SetDlgItemText(hDlg, IDC_CHECK3, "OK");
	uf=uifile1;
	for (;;) {
		FILE *f;
		if (uf==NULL)
			break;
		ufn=uf->next;
		f=fopen(uf->path, "rb");
		if (f!=NULL) {
			fclose(f);
			if (!DeleteFile(uf->path)) {
				if (rf==0) {
					SendDlgItemMessage(hDlg, IDC_LIST1, LB_ADDSTRING, 0, (LPARAM)"***Following files weren't removed!***");
					rf=1;
				}
				SendDlgItemMessage(hDlg, IDC_LIST1, LB_ADDSTRING, 0, (LPARAM)uf->path);
			}
		}
		delete uf;
		uf=ufn;
	}
	SetDlgItemText(hDlg, IDC_CHECK1, "OK");
	ud=uidire;
	for (;;) {
		if (ud==NULL)
			break;
		udn=ud->prev;
		if (SetCurrentDirectory(ud->path)) {
			SetCurrentDirectory("..");
			if (!RemoveDirectory(ud->path)) {
				if (rd==0) {
					SendDlgItemMessage(hDlg, IDC_LIST1, LB_ADDSTRING, 0, (LPARAM)"***Following directories weren't removed!***");
					rd=1;
				}
				SendDlgItemMessage(hDlg, IDC_LIST1, LB_ADDSTRING, 0, (LPARAM)ud->path);
			}
		}
		delete ud;
		ud=udn;
	}
	SetDlgItemText(hDlg, IDC_CHECK2, "OK");
	DeleteFile(log);
	SetDlgItemText(hDlg, IDC_CHECK4, "OK");
	EnableWindow(GetDlgItem(hDlg, IDCANCEL), TRUE);
	uithread=NULL;
	return 0;
}

BOOL APIENTRY UninstDlg(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
	DWORD ti;
	switch(wMsg) {
		case WM_INITDIALOG:
			uithread=CreateThread(NULL, 0, uninstall_program, (LPVOID)hDlg, CREATE_SUSPENDED, &ti);
			return TRUE;
		case WM_COMMAND:
			switch (LOWORD(wParam)) {
			case IDCANCEL:
				EndDialog(hDlg, 0);
				return 0;
			case IDOK:
				EnableWindow(GetDlgItem(hDlg, IDOK), FALSE);
				EnableWindow(GetDlgItem(hDlg, IDCANCEL), FALSE);
				if (uithread!=NULL)
					ResumeThread(uithread);
				return 0;
			}
			break;
	}
	return 0;
}